/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2019 Arm Limited (or its affiliates). All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    MassStorage.c
 * Purpose: USB Device - Mass Storage example
 *----------------------------------------------------------------------------*/

#include "main.h"
#include "rl_fs.h"                 /* Keil.MDK-Pro::File System:CORE          */
#include "rl_usb.h"                /* Keil.MDK-Pro::USB:CORE                  */

#include "GLCD_Config.h"           /* Keil.MCB1800::Board Support:Graphic LCD */
#include "GLCD_Fonts.h"

#include "SCU_LPC18xx.h"           /* Keil::Device:SCU                        */


/* Main stack size must be multiple of 8 Bytes */
#define APP_MAIN_STK_SZ (1024U)
static uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};

/*------------------------------------------------------------------------------
 *        Application
 *----------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *arg) {
  (void)arg;

  GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
  GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
  GLCD_ClearScreen        ();
  GLCD_SetFont            (&GLCD_Font_16x24);
  GLCD_DrawString         (0U, 0U*24U, "    USB Device      ");
  GLCD_DrawString         (0U, 1U*24U, "    MSC Class       ");
  GLCD_DrawString         (0U, 2U*24U, "Mass Storage Example");
  GLCD_DrawString         (0U, 4U*24U, "USB0: SD Card Disk  ");
  GLCD_DrawString         (0U, 5U*24U, "USB1: Memory Disk   ");
  GLCD_DrawString         (0U, 8U*24U, "  Keil Tools by ARM ");
  GLCD_DrawString         (0U, 9U*24U, "    www.keil.com    ");

  /* Disable pull-ups on pins used for VBUS host drive signals P6_3 and P9_5  */
  SCU_PinConfigure (6U, 3U, SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_PULLUP_DIS);
  SCU_PinConfigure (9U, 5U, SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_PULLUP_DIS);

  USBD_Initialize         (0U);         /* USB Device 0 Initialization        */
  USBD_Connect            (0U);         /* USB Device 0 Connect               */

  USBD_Initialize         (1U);         /* USB Device 1 Initialization        */
  USBD_Connect            (1U);         /* USB Device 1 Connect               */

  osThreadExit();
}
